import { __decorate } from "tslib";
import { NumberField as FASTNumberField } from '@microsoft/fast-components';
import { attr } from '@microsoft/fast-element';
import { foundationNumberFieldStyles as styles } from './number-field.styles';
import { foundationNumberFieldTemplate as template } from './number-field.template';
const formattedInputPrecision = 12;
export class NumberField extends FASTNumberField {
    constructor() {
        super(...arguments);
        this.hideStep = false;
        this.value = '';
        this.withFormatting = false;
        this.isStepChange = false;
    }
    sanitiseInputVal(inputVal) {
        return inputVal.replace(/[^0-9\-+.]|\b[-+]|[.]{2,}$/g, '');
    }
    toIntlNumberFormat(value) {
        return Intl.NumberFormat().format(Number(value.replace(/,/g, '')));
    }
    validateValue(value) {
        var _a, _b;
        let validValue = parseFloat(parseFloat(this.sanitiseInputVal(value)).toPrecision(formattedInputPrecision));
        if (isNaN(validValue)) {
            validValue = '';
        }
        else {
            validValue = Math.min(validValue, (_a = this.max) !== null && _a !== void 0 ? _a : validValue);
            validValue = Math.max(validValue, (_b = this.min) !== null && _b !== void 0 ? _b : validValue).toString();
        }
        return validValue.toLocaleString();
    }
    handleStepChange() {
        this.isStepChange = true;
        this.value = this.sanitiseInputVal(this.value);
        this.isStepChange = false;
    }
    stepUp() {
        this.handleStepChange();
        super.stepUp();
    }
    stepDown() {
        this.handleStepChange();
        super.stepDown();
    }
    valueChanged(previous, next) {
        if (this.isStepChange) {
            return;
        }
        if (this.withFormatting) {
            const rgxIsNegativeOrDecimal = /^(?![\s\S])|^-$|\.0*$/g;
            this.value = rgxIsNegativeOrDecimal.test(next)
                ? next
                : this.toIntlNumberFormat(this.validateValue(next));
        }
        else {
            super.valueChanged(previous, this.value);
        }
    }
}
__decorate([
    attr({ mode: 'boolean' })
], NumberField.prototype, "hideStep", void 0);
__decorate([
    attr
], NumberField.prototype, "value", void 0);
__decorate([
    attr({ mode: 'boolean' })
], NumberField.prototype, "withFormatting", void 0);
export const foundationNumberFieldShadowOptions = {
    delegatesFocus: true,
    mode: 'open',
};
export const defaultNumberFieldConfig = {
    stepDownGlyph: `
    <svg class="step-down-glyph" viewBox="0 0 24 24" width="1rem" height="1rem" xmlns="http://www.w3.org/2000/svg">
      <path d="M6.1018 8C5.02785 8 4.45387 9.2649 5.16108 10.0731L10.6829 16.3838C11.3801 17.1806 12.6197 17.1806 13.3169 16.3838L18.8388 10.0731C19.5459 9.2649 18.972 8 17.898 8H6.1018Z"/>
    </svg>
  `,
    stepUpGlyph: `
    <svg class="step-up-glyph" viewBox="0 0 24 24" width="1rem" height="1rem" xmlns="http://www.w3.org/2000/svg">
      <path d="M6.1018 16.9814C5.02785 16.9814 4.45387 15.7165 5.16108 14.9083L10.6829 8.59762C11.3801 7.80079 12.6197 7.80079 13.3169 8.59762L18.8388 14.9083C19.5459 15.7165 18.972 16.9814 17.898 16.9814H6.1018Z"/>
    </svg>
  `,
};
export const foundationNumberField = NumberField.compose(Object.assign({ baseName: 'number-field', template,
    styles, shadowOptions: foundationNumberFieldShadowOptions }, defaultNumberFieldConfig));
